import 'dart:math';

extension ListX<T> on List<T> {
  T? itemOrNull(int at) {
    if (isNotEmpty && 0 <= at && at < length) {
      return this[at];
    }
    return null;
  }

  T? get firstOrNull => itemOrNull(0);

  T? get lastOrNull => itemOrNull(length - 1);

  T? firstMatchOrNull(bool Function(T model) matcher) {
    for (var model in this) {
      if (matcher(model)) {
        return model;
      }
    }
    return null;
  }

  List<E> mapIndexed<E>(
    E Function(int index, T model) mapper,
  ) =>
      asMap()
          .map((key, value) => MapEntry(key, mapper(key, value)))
          .values
          .toList();

  void forEachIndexed(
    Function(int index, T model) lopper,
  ) {
    asMap().forEach(lopper);
  }

  List<T> dropFirst([int first = 1]) => sublist(0, first);

  List<T> dropLast([int last = 1]) => sublist(0, max(length, length - last));
}

extension FlattableList<T> on Iterable<Iterable<T>> {
  Iterable<E> flatMap<E>(E Function(T item) mapper) => flatten().map(mapper);
  Iterable<T> flatten() => expand((element) => element);
}

extension LixtX<T> on Iterable<T> {
  Iterable<E> compactMap<E>(E? Function(T item) mapper) =>
      map(mapper).filterNull();
}

extension NullableListX<T> on Iterable<T?> {
  Iterable<T> filterNull() =>
      where((element) => element != null).map((e) => e as T);
}
