import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:getx_extended/export.dart';

class RxTabBar<T> extends StatelessWidget implements PreferredSizeWidget {
  Rx<int> page;
  List<T> source;
  Widget Function(T item, bool isSelected) builder;
  bool isScrollable;
  Color indicatorColor;
  double inactiveOpacity;
  double? height;
  Alignment? alignment;
  Decoration? indicator;
  ValueChanged<int>? onTap;

  RxTabBar({
    required this.source,
    required this.builder,
    required this.page,
    this.isScrollable = true,
    this.indicatorColor = Colors.transparent,
    this.inactiveOpacity = 0.3,
    this.alignment,
    this.height,
    this.indicator,
    this.onTap,
  });

  @override
  Widget build(BuildContext context) {
    final child = source.isEmpty
        ? Container()
        : TabBar(
            onTap: onTap,
            tabs: [
              ...source.mapIndexed(
                (index, e) => page.toWidget(
                  (rx) => Opacity(
                    opacity: rx.value == index ? 1 : inactiveOpacity,
                    child: builder(e, rx.value == index),
                  ),
                ),
              )
            ],
            indicator: indicator,
            indicatorColor: indicatorColor,
            isScrollable: isScrollable,
          );
    if (alignment != null) {
      return Align(
        alignment: alignment!,
        child: child,
      );
    }
    return child;
  }

  @override
  Size get preferredSize => Size.fromHeight(
        source.isEmpty ? 0 : (height ?? kToolbarHeight),
      );
}
