import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:getx_extended/export.dart';

class RxPageableList<Request, Response> extends StatelessWidget {
  Widget Function(Response item) builder;
  RxQueryPageable<Request, Response> source;
  bool reverse;

  Widget? header;
  Widget? separator;
  Widget? empty;
  bool shrinkWrap;
  EdgeInsetsGeometry? padding;
  VoidCallback? next;
  Axis scrollDirection;

  RxPageableList({
    required this.builder,
    required this.source,
    this.scrollDirection = Axis.vertical,
    this.reverse = false,
    this.shrinkWrap = false,
    this.header,
    this.separator,
    this.next,
    this.padding,
    this.empty,
  });

  @override
  Widget build(BuildContext context) => ObxValue<RxList<Response>>(
        (rx) => Stack(
          children: [
            _body(rx),
            _empty(rx, empty),
          ],
        ),
        source,
      );

  Widget _body(RxList<Response> rx) {
    final offset = header != null ? 1 : 0;
    return ListView.separated(
      padding: padding,
      reverse: reverse,
      shrinkWrap: shrinkWrap,
      scrollDirection: scrollDirection,
      physics: shrinkWrap ? NeverScrollableScrollPhysics() : null,
      itemBuilder: (_, index) {
        if (header != null && index == 0) {
          return header!;
        }
        final realIndex = index - offset;
        return builder(source.item(realIndex));
      },
      separatorBuilder: (_, __) => separator ?? Container(),
      itemCount: rx.length + offset,
    );
  }

  late TaskController _task = Get.find();

  Widget _empty(
    List<Response> list,
    Widget? empty,
  ) =>
      _task.isProcessing.toWidget(
        (rx) => !rx.value && list.isEmpty
            ? FutureBuilder<bool>(
                initialData: false,
                future: Future.delayed(Duration(milliseconds: 500))
                    .then((_) => true),
                builder: (_, s) =>
                    s.data == true ? empty ?? Container() : Container(),
              )
            : Container(),
      );
}
